#!/usr/bin/python
#----------------------------
# Config section
#----------------------------

#Define the directory containing the programs to be run (relative to this script)
programsDir="./bin/"
#Define the directory containing the input files (relative to this script)
inputDir="./in/"
#Define the directory containing the input files (relative to this script)
outputDir="./out/"
#Set the list of programs to be run
programs=["run_tetra_psync_strips","run_tetra_psync_blocks"]

#Define the sizes
#sizes = {1: [128, 200, 256, 400, 512, 600, 700, 1024, 1200, 1400, 1600, 1800, 2048, 3000, 4000, 5000, 6000, 7000, 8000, 9000, 10000, 11000, 12000, 13000, 14000, 15000, 16000, 23000, 28000], 2: [128, 200, 256, 400, 512, 600, 700, 1024, 1200, 1400, 1600, 1800, 2048, 3000, 4000, 5000, 6000, 7000, 8000, 9000, 10000, 11000, 12000, 13000, 14000, 15000, 16000, 23000, 28000, 33000, 36000, 41000, 44000], 4: [128, 200, 256, 400, 512, 600, 700, 1024, 1200, 1400, 1600, 1800, 2048, 3000, 4000, 5000, 6000, 7000, 8000, 9000, 10000, 11000, 12000, 13000, 14000, 15000, 16000, 23000, 28000, 33000, 36000, 41000, 44000, 47000, 50000, 52000]}
sizes = {2: [1024, 10000, 44000], 4: [1024, 10000, 44000, 52000]}

#Define the shapes as dimensional factors: 
#With size s and factors [f,g] -> [s*f, s*g] matrix
#shapes = {1: [[1,1]], 2: [[1,1]], 4: [[0.5,1],[1,1],[1,0.5]]}
#shapes = {1: [[1,1]], 2: [[1,1]], 4: [[1,1],[1,0.5]]}
shapes = {1: [[1,1]], 2: [[1,1]], 4: [[1,1]]}

#Define the number of GPUs to use relatively to the program index
#gpusList = {0: [1,2,4], 1: [4]}
gpusList = {0: [2,4], 1: [4]}

bsList = [1, 2, 4, 5, 8, 10, 20, 25, 40, 50, 100]
#Define the relaxation factor omega (0 = use the optimum value)
omega=0
#Save graphics output
img=0

#-----------------------------------------------
# Do not edit beyond this line
#-----------------------------------------------
from subprocess import *
import re
import sys
import math
import time

################################################

def setupOutFile(outFileName):
    
    outFile=file(outputDir+outFileName,"w")
    outFile.write("M;N;Border_Size;MKT[ms];MST[ms];MTT[ms];MCT[ms];Elapsed Time[s]\n")

    return outFile

################################################

def runTetra(program, gpus, m, n, bs, k):
    
    #Prepare the extraction regexp
    mktREs=[]
    mstREs=[]
    mttREs=[]
    mctREs=[]
    for gpu in range(gpus):
        mktREs.append(re.compile("t"+str(gpu)+" MKT: (\d+.\d+)"))
        mstREs.append(re.compile("t"+str(gpu)+" MST: (\d+.\d+)"))
        mttREs.append(re.compile("t"+str(gpu)+" MTT: (\d+.\d+)"))
        mctREs.append(re.compile("t"+str(gpu)+" MCT: (\d+.\d+)"))
    elapsedRE=re.compile("Elapsed time: (\d+.\d+)")
    
    dim = n;
    if (dim < m): dim = m;
      
    int_iter=100
    if (dim >= 33000):
       ext_iter=5
    elif (dim >= 8000) and (n < 33000):
       ext_iter=10
    elif (dim >= 4000) and (n < 8000):
       ext_iter=20
    elif (dim >= 128) and (n < 4000):
       ext_iter=30

    resList=[]
    cmdLine=programsDir + program + " " + str(gpus) + " " + str(m) + " " + str(n) + " " +inputDir+str(m)+"."+str(n)+".in"+ " " + str(int_iter) + " " + str(bs) + " " + str(k) + " " + str(omega) + " " + str(img)
    
    ttime=0
    
    for i in range(ext_iter):
        mkts = []
        msts = []
        mtts = []
        mcts = []
        start = time.time()
        p = Popen(cmdLine, shell=True, stdin=PIPE, stdout=PIPE, close_fds=True)
        p.wait()
        output = p.stdout.read()
        elapsed = time.time()-start
        if p.returncode != 0:
           print "Run #"+str(i)+" - Something went wrong at this stage. Return code " + str(p.returncode)
           exit(1)
        else:
           if elapsed > 60:
              print "Run #%d - Execution went fine in %.2fmin. Extracting relevant data..."  % (i, elapsed/60)
           else:
              print "Run #%d - Execution went fine in %.2fs. Extracting relevant data..."  % (i, elapsed)
        ttime += elapsed
        #Extract
        for gpu in range(gpus):
            mkts.append(float(mktREs[gpu].search(output).group(1)))
            msts.append(float(mstREs[gpu].search(output).group(1)))
            mtts.append(float(mttREs[gpu].search(output).group(1)))
            mcts.append(float(mctREs[gpu].search(output).group(1)))
        resList.append([]);
        resList[i].append(max(mkts))
        resList[i].append(max(msts))
        resList[i].append(max(mtts))
        resList[i].append(max(mcts))
        resList[i].append(float(elapsedRE.search(output).group(1)))
    final=ttime/60
    if final > 60: 
       print "Done %d runs in %.2fh. Saving result..." % (ext_iter, final/60)
    else:
       print "Done %d runs in %.2fmin. Saving result..." % (ext_iter, ttime/60)

    return [ext_iter, int_iter, resList]        
    
################################################

def main():
    if len(sys.argv) < 2 or len(sys.argv) > 3:
       print "Usage 1: launcher <border_size>"
       print "Usage 2: launcher <border_size_inf> <border_size_sup>"
       exit(1)
    elif len(sys.argv) == 2:
         sup = inf = int(sys.argv[1])
    else:
         inf = int(sys.argv[1])
         sup = int(sys.argv[2])

    if inf < 0 or sup < inf:
       print "Need 0 <= inf <= sup"
       exit(1)

    for p in [0,1]:
        for k in [0,1]:
            for gpus in gpusList[p]:
                for shape in shapes[gpus]:
                    print "Preparing the output file..."
                    outFile=setupOutFile("res_"+programs[p]+"_k"+str(k)+"_g"+str(gpus)+"_s"+str(shapes[gpus].index(shape)+1)+".csv")
                    print "Done"
                    for size in sizes[gpus]:
                        for bs in bsList:
                            m = int(shape[0]*size)
                            n = int(shape[1]*size)
                            print "Execution of: " + programs[p] + "[GPUs=" + str(gpus) + ",M=" + str(m) + ",N=" + str(n) + ",K="+ str(k) + ",bs="+ str(bs) + ",omega=" + str(omega) + ",saveImg=" + str(img) +"]"
                            result=runTetra(programs[p], gpus, m, n, bs, k)
                            for res in result[2]:
                                outFile.write(str(m)+"\t"+str(n)+"\t"+str(bs)+"\t"+str(res[0])+"\t"+str(res[1])+"\t"+str(res[2])+"\t"+str(res[3])+"\t"+str(res[4])+"\n")
                                outFile.flush()
                            print "Saved."
                    outFile.close()
    print "Stop"

#--------------------------------------------------
#Execution starts here
#--------------------------------------------------
if __name__=="__main__":
    main()
    



