#include <stdio.h>
#include <pde_kernels.cu>

int main(int argc, char **argv)
{
	cudaEvent_t start, stop;
	float time;
	
	cudaEventCreate(&start);
	cudaEventCreate(&stop);

	int n = atoi(argv[1]);
		
	size_t pitch;
	
	float *dgrid;
	float *hgrid = (float *)malloc(n*n*sizeof(float));
	cudaMallocPitch((void **)&dgrid, &pitch, n*sizeof(float), n);
	
	//Compute the time required to send 0Byte (Startup)
	float startup;
	cudaEventRecord(start, 0);

	cudaMemcpy2D(dgrid, pitch, hgrid, n*sizeof(float), 0, 0, cudaMemcpyHostToDevice);

	cudaEventRecord(stop, 0);
	cudaEventSynchronize(stop);
	cudaEventElapsedTime(&startup, start, stop);
	printf("CudaMemcpy2D startup: %fms\n", startup);
	
	//Compute the time required to send 1Byte
	float mean = 0.f;
	for(int i=0; i<1000; i++)
	{
		cudaEventRecord(start, 0);

		cudaMemcpy2D(dgrid, pitch, hgrid, n*sizeof(float), 1, 1, cudaMemcpyHostToDevice);

		cudaEventRecord(stop, 0);
		cudaEventSynchronize(stop);
		cudaEventElapsedTime(&time, start, stop);
		mean += time;
	}
	float tword = mean/1000-startup;
	printf("CudaMemcpy2D 1 Byte: %fms\n", tword);
	
	//Time required to transfer n/2*n elements
	cudaEventRecord(start, 0);

	cudaMemcpy2D(dgrid, pitch, hgrid, n*sizeof(float), n*sizeof(float), n/2, cudaMemcpyHostToDevice);

	cudaEventRecord(stop, 0);
	cudaEventSynchronize(stop);
	cudaEventElapsedTime(&time, start, stop);
	printf("CudaMemcpy2D to send %d x %d/2: %fms\n", n, n, time);
	
	//Time required to transfer n*n/2 elements
	cudaEventRecord(start, 0);

	cudaMemcpy2D(dgrid, pitch, hgrid+n*n/2, n*sizeof(float), n*sizeof(float)/2, n, cudaMemcpyHostToDevice);

	cudaEventRecord(stop, 0);
	cudaEventSynchronize(stop);
	cudaEventElapsedTime(&time, start, stop);
	printf("CudaMemcpy2D to send %d x %d/2: %fms\n", n, n, time);
	
	//Time required to transfer n² elements
	cudaEventRecord(start, 0);

	cudaMemcpy2D(dgrid, pitch, hgrid, n*sizeof(float), n*sizeof(float), n, cudaMemcpyHostToDevice);

	cudaEventRecord(stop, 0);
	cudaEventSynchronize(stop);
	cudaEventElapsedTime(&time, start, stop);
	printf("CudaMemcpy2D to send %d x %d: %fms\n", n, n, time);
	
	cudaFree(dgrid);
	cudaEventDestroy(start);
	cudaEventDestroy(stop);
	
	free(hgrid);
	
	return 0;
}
